/**
 * Checks if the given object is a valid number.
 *
 * \return true if the given object is a number and not NaN or +/-Infinity
 * or NULL or undefined.
 */
function isNumber(obj) {
	return (typeof (obj) === "number" && !isNaN(obj) && isFinite(obj));
};

/**
 * Checks if the given object is undefined or null.
 *
 * \return true if the given object is defined and not null.
 * This method also supports shared pointers, i.e. obj.isNull()==true.
 */
function isNull(obj) {
	if (isDeleted(obj)) {
		return true;
	}
	return (obj == null || (typeof (obj.isNull) === "function" && obj.isNull() === true));
};

/**
 * Checks if the given object is of the given type. E.g.
 * isOfType(myObject, QLabel) returns true if myObject is a QLabel. Note
 * that isOfType(myObject, QWidget) returns false even though QLabel is
 * derived from QWidget.
 */
function isOfType(obj, type) {
	return !isNull(obj) && obj.constructor === type;
};

/**
 * Checks if the given object is a hatch entity.
 *
 * \return true if the given object is an arc entity (RHatchEntity).
 */
function isHatchEntity(obj) {
	return isOfType(obj, RHatchEntity) || isOfType(obj, RHatchEntityPointer);
};

/**
 * Checks if the given object is a line entity.
 *
 * \return true if the given object is a line entity (RLineEntity).
 */
function isLineEntity(obj) {
	return isOfType(obj, RLineEntity) || isOfType(obj, RLineEntityPointer);
};

/**
 * Checks if the given object is a block reference entity.
 *
 * \return true if the given object is a block reference entity (RBlockReferenceEntity).
 */
function isBlockReferenceEntity(obj) {
	return isOfType(obj, RBlockReferenceEntity)
			|| isOfType(obj, RBlockReferenceEntityPointer);
};

/**
 * Checks the type of the given object.
 *
 * \return true if the given object is a combo box.
 */
function isComboBox(obj) {
	return !isNull(obj)
			&& (isOfType(obj, QComboBox) ||
			    isOfType(obj, RColorCombo) || 
				isOfType(obj, RLineweightCombo) || 
				isOfType(obj, RLinetypeCombo));
};

function isLeapYear(year) {
	if (year % 4 == 0) {
		if (year % 100 != 0) {
			return true;
		} else {
			if (year % 400 == 0)
				return true;
			else
				return false;
		}
	}
	return false;
};

function createBlock(document, di) {
	var block = new RBlock(document, "Weekend", new RVector(0, 0));
	var op = new RAddObjectOperation(block);
	di.applyOperation(op);

	// make new block the current block:
	di.setCurrentBlock("Weekend");

	// add hatch to the block definition
	var p1 = new RVector(0, 0);
	var p2 = new RVector(93, 0);
	var p3 = new RVector(93, 25);
	var p4 = new RVector(0, 25);

	var line1 = new RLineEntity(document, new RLineData(p1, p2));
	var line2 = new RLineEntity(document, new RLineData(p2, p3));
	var line3 = new RLineEntity(document, new RLineData(p3, p4));
	var line4 = new RLineEntity(document, new RLineData(p4, p1));
	var hatchData = new RHatchData(true, 1.0, 0.0, "SOLID");
	hatchData.newLoop();
	hatchData.addBoundary(line1.getData().castToShape());
	hatchData.addBoundary(line2.getData().castToShape());
	hatchData.addBoundary(line3.getData().castToShape());
	hatchData.addBoundary(line4.getData().castToShape());

	var hatch = new RHatchEntity(document, hatchData);
	op = new RAddObjectsOperation();
	op.addObject(hatch);
	var transaction = di.applyOperation(op);
	var ids = transaction.getAffectedObjects();
	op = new RModifyObjectsOperation();
	for (var i = 0; i < ids.length; i++) {
		var id = ids[i];
		var entity = document.queryEntity(id);
		if (isHatchEntity(entity)) {
			// Change hatch colour
			entity.setColor(new RColor(190, 190, 190));
			op.addObject(entity, false);
		}
	}
	di.applyOperation(op);

	// switch back to main drawing (block "*Model_Space"):
	di.setCurrentBlock(RBlock.modelSpaceName);
};

function drawGridOutline(document, di) {
	// corner points of the grid
	var p1 = new RVector(7, 782);
	var p2 = new RVector(7, 7);
	var p3 = new RVector(1181, 7);
	var p4 = new RVector(1181, 833);
	var p5 = new RVector(65, 833);

	var op = new RAddObjectsOperation();
	// add line entities
	var line1 = new RLineEntity(document, new RLineData(p1, p2));
	var line2 = new RLineEntity(document, new RLineData(p2, p3));
	var line3 = new RLineEntity(document, new RLineData(p3, p4));
	var line4 = new RLineEntity(document, new RLineData(p4, p5));
	op.addObject(line1);
	op.addObject(line2);
	op.addObject(line3);
	op.addObject(line4);
	var p5 = new RVector(65, 7);
	var p6 = new RVector(65, 833);
	var line5 = new RLineEntity(document, new RLineData(p5, p6));
	op.addObject(line5);
	var p7 = new RVector(65, 807);
	var p8 = new RVector(1181, 807);
	var line6 = new RLineEntity(document, new RLineData(p7, p8));
	op.addObject(line6);
	var p9 = new RVector(7, 782);
	var p10 = new RVector(1181, 782);
	var line7 = new RLineEntity(document, new RLineData(p9, p10));
	op.addObject(line7);
	var transaction = di.applyOperation(op);
	var ids = transaction.getAffectedObjects();
	op = new RModifyObjectsOperation();
	for (var i = 0; i < ids.length; i++) {
		var id = ids[i];
		var entity = document.queryEntity(id);
		if (isLineEntity(entity)) {
			// Change lineweight of these lines
			entity.setLineweight(RLineweight.Weight070);
			op.addObject(entity, false);
		}
	}
	di.applyOperation(op);
};

function drawVerticalLines(document, di) {
	var boxWidth = new RVector(93, 0);
	var boxHeight = new RVector(0, 25);

	var p1 = new RVector(65, 7);
	var p2 = new RVector(65, 807);
	var op = new RAddObjectsOperation();

	// Draw vertical lines
	for (var i = 0; i < 11; i += 1) {
		p1 = p1.operator_add(boxWidth);
		p2 = p2.operator_add(boxWidth);
		op.addObject(new RLineEntity(document, new RLineData(p1, p2)));
	}
	di.applyOperation(op);
};

function drawHorizontalLines(document, di) {
	var boxWidth = new RVector(93, 0);
	var boxHeight = new RVector(0, 25);

	var p1 = new RVector(7, 7);
	var p2 = new RVector(1181, 7);
	var op = new RAddObjectsOperation();

	// Draw horizontal lines
	for (var i = 0; i < 30; i += 1) {
		p1 = p1.operator_add(boxHeight);
		p2 = p2.operator_add(boxHeight);
		op.addObject(new RLineEntity(document, new RLineData(p1, p2)));
	}
	di.applyOperation(op);
};

function drawDayNumbers(document, di) {
	var p1 = new RVector(57, 769.5);
	var boxHeight = new RVector(0, 25);
	var daydata = new RTextData(p1, p1, 18, 1.0, RS.VAlignMiddle,
			RS.HAlignRight, RS.LeftToRight, RS.Exact, 1.0, "1", "Arial", true,
			false, 0.0, false);
	var op = new RAddObjectsOperation();
	for (var i = 1; i < 32; i += 1) {
		daydata.setText(i.toString());
		daydata.setPosition(p1);
		daydata.setAlignmentPoint(p1);
		op.addObject(new RTextEntity(document, daydata));
		p1 = p1.operator_subtract(boxHeight);
	}
	di.applyOperation(op);
};

function drawYear(document, di, date) {
	// Add year to grid
	var year = date.getFullYear();
	var p1 = new RVector(72, 820.5);
	var yeardata = new RTextData(p1, p1, 18, 1.0, RS.VAlignMiddle,
			RS.HAlignLeft, RS.LeftToRight, RS.Exact, 1.0, year.toString(),
			"Arial", true, false, 0.0, false);
	var op = new RAddObjectsOperation();
	op.addObject(new RTextEntity(document, yeardata));
	di.applyOperation(op);

	// add next year (if any)
	if (date.getMonth() > 0) {
		year = date.getFullYear() + 1;
		var pos = ((12 - date.getMonth()) * 93) + 65 + 7;
		var p2 = new RVector(pos, 820.5);
        op = new RAddObjectsOperation();
		yeardata.setText(year.toString());
		yeardata.setPosition(p2);
		yeardata.setAlignmentPoint(p2);
		op.addObject(new RTextEntity(document, yeardata));
		var p3 = new RVector(pos - 7, 7);
		var p4 = new RVector(pos - 7, 833);
		op.addObject(new RLineEntity(document, new RLineData(p3, p4)));
		var transaction = di.applyOperation(op);
		var ids = transaction.getAffectedObjects();
        op = new RModifyObjectsOperation();
		for (var i = 0; i < ids.length; i++) {
			var id = ids[i];
			var entity = document.queryEntity(id);
			if (isLineEntity(entity)) {
				// Change lineweight between december and january
				entity.setLineweight(RLineweight.Weight070);
				op.addObject(entity, false);
			}
		}
		di.applyOperation(op);
	}
};

function drawMonths(document, di, date) {
	var boxWidth = new RVector(93, 0);
	var boxHeight = new RVector(0, 25);

	// Add month headings to grid
	var months = ['Ene', 'Feb', 'Mar', 'Abr', 'May', 'Jun'​​, 'Jul', 'Ago', 'Sep', 'Oct', 'Nov', 'Dic'];
	var month = months[date.getMonth()];
	var p1 = new RVector(111.5, 794.5);
	var monthdata = new RTextData(p1, p1, 18, 1.0, RS.VAlignMiddle,
			RS.HAlignCenter, RS.LeftToRight, RS.Exact, 1.0, month.toString(),
			"Arial", true, false, 0.0, false);
	var op = new RAddObjectsOperation();
	for (var i = date.getMonth(); i < 12; i += 1) {
		monthdata.setText(months[i]);
		monthdata.setPosition(p1);
		monthdata.setAlignmentPoint(p1);
		p1 = p1.operator_add(boxWidth);
		op.addObject(new RTextEntity(document, monthdata));
	}
	for (i = 0; i < date.getMonth(); i += 1) {
		monthdata.setText(months[i]);
		monthdata.setPosition(p1);
		monthdata.setAlignmentPoint(p1);
		p1 = p1.operator_add(boxWidth);
		op.addObject(new RTextEntity(document, monthdata));
	}
	di.applyOperation(op);
};

function drawDayNames(document, di, date) {
	var boxWidth = new RVector(93, 0);
	var boxHeight = new RVector(0, 25);
	var year = date.getFullYear();

	// Add day names to grid
	var dayNames = ['Domingo', 'Lunes', 'Martes', 'Miércoles','Jueves','Viernes','Sábado'];
	var daysInMonth = [ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ];
	// check for leap year
	if (date.getMonth() > 1) {
		year = date.getFullYear() + 1;
	} else {
		year = date.getFullYear();
	}
	if (isLeapYear(year)) {
		daysInMonth[1] = 29;
	}
	//----------------------------------------------------
	var day = date.getDay();
	var dayname = dayNames[day];
	var p12 = new RVector(67, 780);
	var wknd = new RVector(-2, -23);
	var daydata = new RTextData(p12, p12, 4, 1.0, RS.VAlignTop, RS.HAlignLeft,
			RS.LeftToRight, RS.Exact, 1.0, dayname.toString(), "Arial", false,
			true, 0.0, false);
	var mnth = date.getMonth();
	var operation = new RAddObjectsOperation();
	for (var m = 0; m < 12; m += 1) {
		for (var d = 0; d < daysInMonth[mnth]; d += 1) {
			daydata.setText(dayname.toString());
			daydata.setPosition(p12);
			daydata.setAlignmentPoint(p12);
			operation.addObject(new RTextEntity(document, daydata));
			// shade saturday and sunday
			if (day === 0 || day === 6) {
				var position = p12.operator_add(wknd);
				var scale = new RVector(1, 1);
				var angle = 0;
				var blockId = document.getBlockId("Weekend");
				var blockRef = new RBlockReferenceEntity(
						document,
						new RBlockReferenceData(blockId, position, scale, angle));
				var op = new RAddObjectsOperation();
				op.addObject(blockRef);
				// move block to back
				var transaction = di.applyOperation(op);
				var ids = transaction.getAffectedObjects();
                op = new RModifyObjectsOperation();
				for (var i = 0; i < ids.length; i++) {
					var id = ids[i];
					var entity = document.queryEntity(id);
					if (isBlockReferenceEntity(entity)) {
						entity.setDrawOrder(-1);
						op.addObject(entity, false);
					}
				}
				di.applyOperation(op);
			}
			if (day >= 6) {
				day = 0;
			} else {
				day = day + 1;
			}
			dayname = dayNames[day];
			p12 = p12.operator_subtract(boxHeight);
		}
		p12.y = 780;
		p12 = p12.operator_add(boxWidth);
		mnth = mnth + 1;
		if (mnth > 11) {
			mnth = 0;
		}
	}
	di.applyOperation(operation);
};

function getYearArg(document, di, date) {
	var yearDialog = new QInputDialog();
	yearDialog.setInputMode(QInputDialog.IntInput);
	yearDialog.setIntRange(1900, 2199);
	yearDialog.setIntStep(1);
	yearDialog.setIntValue(date.getFullYear());
	yearDialog.setLabelText("Año de inicio:");
	yearDialog.setWindowTitle("Wallchart - Año de inicio");
	if (!yearDialog.exec()) {
		yearDialog.destroy();
	}
	return yearDialog.intValue();
};

function getMonthArg(document, di, date) {
	var months = "Enero, Febrero, Marzo, Abril, Mayo, Junio​​, Julio, Agosto, Septiembre, Octubre, Noviembre, Diciembre";
	var items = months.split(",");
	var monthDialog = new QInputDialog();
	monthDialog.setInputMode(3); // getItem input (4 also works?)
	monthDialog.setComboBoxItems(items);
	monthDialog.setComboBoxEditable(false);
	var children = monthDialog.children();
	for (var i = 0; i < children.length; i++) { // find combo box and set index to month
		var c = children[i];
		if (isComboBox(c)) {
			c.setCurrentIndex(date.getMonth());
			break;
		}
	}
	monthDialog.setLabelText("Comience Mes:");
	monthDialog.setWindowTitle("Wallchart - Comience Mes");
	if (!monthDialog.exec()) {
		monthDialog.destroy();
	}
	var txt = monthDialog.textValue();
	var idx = items.indexOf(txt, 0);
	return idx;
};

// This is the main function that draws the wallchart.
function Wallchart_en() {
	var appWin = EAction.getMainWindow();
	var document = appWin.getDocument();
	var di = appWin.getDocumentInterface();

	var date = new Date();
	var year = getYearArg(document, di, date);
	var month = getMonthArg(document, di, date);

	if (isNumber(year)) {
		date.setFullYear(year);
	}
	if (isNumber(month)) {
		date.setMonth(month);
	}
	date.setDate(1);

	createBlock(document, di); // create block definition to highlight weekends
	drawGridOutline(document, di);
	di.autoZoom();
	drawVerticalLines(document, di);
	drawHorizontalLines(document, di);
	drawDayNumbers(document, di);
	drawYear(document, di, date);
	drawMonths(document, di, date);
	drawDayNames(document, di, date);
};

//------------------------------------------------------------------
// Wallchart
//
//------------------------------------------------------------------

Wallchart_en();
